@@ -9,20 +9,26 @@ AlbumView::AlbumView(Spotify::Session* pSession, QWidget *pParent)
 
     setWindowTitle(tr("Browse Album"));
 
+    m_AlbumReviewDlg.setWindowTitle(tr("Album Review"));
+
     m_pAlbumBrowser = NULL;
 
     m_CallerID = ALBUMVIEW_ID;
 
     m_pSession = pSession; //set Spotify session instance pointer
 
+    m_pAlbumCoverContainer = new Spotify::ImageContainer(this); //allocate coverart imagecontainer instance
+
     //setup signal / slot connections
+    connect(m_pAlbumCoverContainer,SIGNAL(loaded()),SLOT(OnAlbumCoverLoaded()));
     connect(m_pSession,SIGNAL(albumBrowserReady(Spotify::AlbumBrowser*,qint32)),SLOT(OnAlbumBrowserReady(Spotify::AlbumBrowser*,qint32)));
-    connect(m_pSession,SIGNAL(coverArtReady(QImage,qint32)),SLOT(OnAlbumCoverReady(QImage,qint32)));
+    connect(&m_AlbumReviewBtn,SIGNAL(clicked()),SLOT(OnAlbumReviewBtn()));
 
     m_pAlbumTrackView = new QListView(this);
     m_pAlbumTrackDelegate = new ListViewDelegate(this);
     m_pAlbumTrackDelegate->setHeaderFontPointSize(15);
     m_pAlbumTrackDelegate->setSubHeaderFontPointSize(14);
+    m_pAlbumTrackDelegate->setItemTextDisabledColor(QColor(190,190,190)); //set disabled item color (gray)
     m_pAlbumTrackModel = new QStandardItemModel(this);
 
     //connect view to item delegate and model
@@ -36,18 +42,21 @@ AlbumView::AlbumView(Spotify::Session* pSession, QWidget *pParent)
     m_AlbumCaption.setFont(captionFont);
     m_AlbumCaption.setWordWrap(true);
 
-    m_AlbumCoverContainer.setFixedSize(290,290);
+    m_AlbumCover.setFixedSize(ALBUMVIEW_COVERART_WIDTH,ALBUMVIEW_COVERART_HEIGHT);
 
-    m_ArtistViewBtn.setFixedWidth(290);
+    m_ArtistViewBtn.setFixedWidth(ALBUMVIEW_COVERART_WIDTH);
     m_ArtistViewBtn.setDisabled(true); //currently disabled
 
-    m_LeftSubLayout.addWidget(&m_AlbumCoverContainer);
+    m_AlbumReviewBtn.setText(tr("Album Review"));
+
+    m_LeftSubLayout.addWidget(&m_AlbumCover);
     m_LeftSubLayout.addWidget(&m_ArtistViewBtn);
     m_LeftSubLayout.setSpacing(10);
 
-    m_MainLayout.addLayout(&m_LeftSubLayout,0,0,2,1);
+    m_MainLayout.addLayout(&m_LeftSubLayout,0,0,3,1);
     m_MainLayout.addWidget(&m_AlbumCaption,0,1,1,1);
-    m_MainLayout.addWidget(m_pAlbumTrackView,1,1,1,1);
+    m_MainLayout.addWidget(&m_AlbumReviewBtn,1,1,1,1);
+    m_MainLayout.addWidget(m_pAlbumTrackView,2,1,1,1);
 
     setLayout(&m_MainLayout);
 }
@@ -63,11 +72,13 @@ void AlbumView::setAlbum(Spotify::Album* pAlbum)
 
     if(m_pSession && pAlbum)
     {
-        m_AlbumCoverContainer.clear();
+        m_AlbumCover.clear();
 
         m_pAlbumTrackModel->clear(); //clear model rows
 
-        m_AlbumCaption.setText("");
+        m_AlbumCaption.clear();
+
+        m_AlbumReviewBtn.setVisible(false);
 
         status = m_pSession->browse(pAlbum,&m_CallerID); //request to browse specified album details
 
@@ -79,6 +90,10 @@ void AlbumView::setAlbum(Spotify::Album* pAlbum)
         else
         {
             //enable progress indication, browse call succeeded
+            if(pAlbum->getYear()!=0)
+                m_AlbumCaption.setText(pAlbum->getName() + " (" + QString::number(pAlbum->getYear()) + ")"); //set album title
+            else
+                m_AlbumCaption.setText(pAlbum->getName()); //set album title (no year)
             setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true);
         }
     }
@@ -109,14 +124,12 @@ void AlbumView::OnAlbumBrowserReady(Spotify::AlbumBrowser* browser, qint32 calle
             {
                 m_pAlbumBrowser = browser;
                 m_pAlbumBrowser->load();
+                if(m_pAlbumBrowser->getReview()!="")
+                    m_AlbumReviewBtn.setVisible(true); //album review available
                 m_pAlbumBrowser->sortByDiscIndex();
                 if(m_pAlbumBrowser->getAlbum())
                 {
-                    m_pSession->getAlbumCover(m_pAlbumBrowser->getAlbum(),&m_CallerID); //request album coverart
-                    if(m_pAlbumBrowser->getAlbum()->getYear()!=0)
-                        m_AlbumCaption.setText(m_pAlbumBrowser->getAlbum()->getName() + " (" + QString::number(m_pAlbumBrowser->getAlbum()->getYear()) + ")"); //set album title
-                    else
-                        m_AlbumCaption.setText(m_pAlbumBrowser->getAlbum()->getName()); //set album title (no year)
+                    m_pSession->loadImage(m_pAlbumBrowser->getAlbum()->getCover(),m_pAlbumCoverContainer); //request album coverart
 
                 }
                 if(m_pAlbumBrowser->getArtist())
@@ -132,6 +145,16 @@ void AlbumView::OnAlbumBrowserReady(Spotify::AlbumBrowser* browser, qint32 calle
     }
 }
 
+void AlbumView::OnAlbumReviewBtn()
+{
+    //display the album review dialog
+    if(m_pAlbumBrowser)
+    {
+        m_AlbumReviewDlg.setText(m_pAlbumBrowser->getReview());
+        m_AlbumReviewDlg.showMaximized();
+    }
+}
+
 void AlbumView::addTracks()
 {
     //add album tracks to the listview
@@ -149,7 +172,14 @@ void AlbumView::addTracks()
         {
             QStandardItem* trackItem = new QStandardItem();
             if(!currTrack->isAvailable())
+            {
                 trackItem->setEnabled(false); //not possible to play unavailable track, but display it anyway in the listview
+                trackItem->setData(-1,ListViewDelegate::customDataRole); //assign -1 as position for disabled track
+            }
+            else
+            {
+               trackItem->setData(pos,ListViewDelegate::customDataRole); //assign position in internal album track list
+            }
             trackItem->setEditable(false);
             trackItem->setData(trackTypeIcon,ListViewDelegate::iconRole);
             trackItem->setData(currTrack->getName(),ListViewDelegate::headerRole);
@@ -158,7 +188,6 @@ void AlbumView::addTracks()
                 trackItem->setData(currTrack->getArtist()->getName(),ListViewDelegate::subHeaderLeftJustifyRole);
             }
             trackItem->setData(currTrack->getDurationAsString(),ListViewDelegate::subHeaderRightJustifyRole);
-            trackItem->setData(pos,Qt::UserRole); //assign position in internal album track list
             m_pAlbumTrackModel->appendRow(trackItem);
         }
     }
@@ -175,15 +204,16 @@ void AlbumView::OnTrackActivated(QModelIndex trackIndex)
 {
     //A track has been activated (e.g., selected) in the list of album tracks
 
-    Q_UNUSED(trackIndex)
+    if(trackIndex.data(ListViewDelegate::customDataRole).toInt()!=-1)
+    {
+        m_pAlbumTrackView->setEnabled(false); //disable listview temporarily
 
-    m_pAlbumTrackView->setEnabled(false); //disable listview temporarily
+        setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true); //show progress indicator
+        setPlayQueue();
+        setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false); //hide progress indicator
 
-    setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true); //show progress indicator
-    setPlayQueue();
-    setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false); //hide progress indicator
-
-    m_pAlbumTrackView->setEnabled(true); //re-enable the listview
+        m_pAlbumTrackView->setEnabled(true); //re-enable the listview
+    }
  }
 
 void AlbumView::setPlayQueue()
@@ -195,13 +225,15 @@ void AlbumView::setPlayQueue()
 
     for(int pos = m_pAlbumTrackView->currentIndex().row(); pos < m_pAlbumTrackModel->rowCount(); pos++)
     {
-        int trackPos = m_pAlbumTrackModel->item(pos)->data(Qt::UserRole).toInt();
-         if(m_pAlbumBrowser) //track from playlist
+        int trackPos = m_pAlbumTrackModel->item(pos)->data(ListViewDelegate::customDataRole).toInt();
+         if(m_pAlbumBrowser && trackPos!=-1) //track from playlist
+        {
              track = m_pAlbumBrowser->getTrack(trackPos);
-         if(track)
-         {
-             if(track->isAvailable()) //only add track if it is available (playable)
-                PlayQueue.append(track);
+             if(track)
+             {
+                 if(track->isAvailable()) //only add track if it is available (playable)
+                     PlayQueue.append(track);
+             }
          }
     }
 
@@ -209,12 +241,10 @@ void AlbumView::setPlayQueue()
     hide();
 }
 
-void AlbumView::OnAlbumCoverReady(QImage image, qint32 callerID)
+void AlbumView::OnAlbumCoverLoaded()
 {
     //Album coverart has been loaded (for current album),
     //assign it to the album coverart container.
-    if(callerID==m_CallerID)
-    {
-         m_AlbumCoverContainer.setPixmap(QPixmap::fromImage(image).scaled(290,290));
-    }
+    if(m_pAlbumCoverContainer)
+        m_AlbumCover.setPixmap(QPixmap::fromImage(m_pAlbumCoverContainer->getImage()).scaled(ALBUMVIEW_COVERART_WIDTH,ALBUMVIEW_COVERART_HEIGHT));
 }
