@@ -14,6 +14,9 @@ QueueView::QueueView(Spotify::Session* pSession, OssoWrapper* pOsso, QWidget* pa
 
     m_pSession = pSession; //set the Spotify session instance
 
+    m_pAlbumCoverContainer = new Spotify::ImageContainer(this); //allocate coverart imagecontainer instance
+    connect(m_pAlbumCoverContainer,SIGNAL(loaded()),SLOT(OnAlbumCoverLoaded()));
+
     m_playing = false; //set initial playback state
 
     m_CallerID = QUEUEVIEW_ID;
@@ -62,7 +65,6 @@ QueueView::QueueView(Spotify::Session* pSession, OssoWrapper* pOsso, QWidget* pa
     //set up signal and slot connections
     connect(m_pSession,SIGNAL(trackBufferingAmountUpdated(qint32)),SLOT(OnTrackBufferedAmountUpdated(qint32)));
     connect(m_pSession,SIGNAL(playbackPositionUpdated(qint64)),SLOT(OnPlaybackPositionUpdated(qint64)));
-    connect(m_pSession,SIGNAL(coverArtReady(QImage,qint32)),SLOT(OnAlbumCoverReady(QImage,qint32)));
     connect(m_pSession,SIGNAL(albumBrowserReady(Spotify::AlbumBrowser*,qint32)),SLOT(OnAlbumBrowseReady(Spotify::AlbumBrowser*,qint32)));
     connect(m_pSession,SIGNAL(playing(bool)),SLOT(OnPlayingState(bool)));
     connect(m_pSession,SIGNAL(playbackFinished()),SLOT(OnTrackPlaybackFinished()));
@@ -74,7 +76,7 @@ QueueView::QueueView(Spotify::Session* pSession, OssoWrapper* pOsso, QWidget* pa
     connect(&m_NextBtn,SIGNAL(clicked()),SLOT(OnNextTrackBtn()));
     connect(menuBar(),SIGNAL(triggered(QAction*)),SLOT(OnMenubarAction(QAction*)));
 
-    m_AlbumCoverContainer.setFixedSize(355,355); //set fixed size of the album coverart
+    m_AlbumCover.setFixedSize(QUEUEVIEW_COVERART_WIDTH,QUEUEVIEW_COVERART_HEIGHT); //set fixed size of the album coverart
 
     //style the player control buttons
     m_PauseResumeBtn.setCheckable(true); //toggle button (between pause / resume states)
@@ -123,7 +125,7 @@ QueueView::QueueView(Spotify::Session* pSession, OssoWrapper* pOsso, QWidget* pa
 
 
     //set up the QueueView UI layout
-    m_GridLayout.addWidget(&m_AlbumCoverContainer,0,0,7,1,Qt::AlignTop);
+    m_GridLayout.addWidget(&m_AlbumCover,0,0,7,1,Qt::AlignTop);
     m_GridLayout.addWidget(m_pTrackInfoView,0,1,7,1);
     m_GridLayout.addLayout(&m_PlayControls,7,0,1,-1);
     m_GridLayout.setContentsMargins(0,5,0,10); //set layout margin
@@ -151,8 +153,7 @@ void QueueView::setupContextMenu()
     connect(pViewAlbumAct,SIGNAL(triggered()),SLOT(OnViewAlbumAction()));
     m_pTrackInfoView->addAction(pViewAlbumAct);
     QAction* pViewArtistAct = new QAction(tr("Browse Artist"),this);
-    pViewArtistAct->setEnabled(false);
-    //connect(pPlayAlbumAct,SIGNAL(triggered()),SLOT(OnViewArtistAction()));
+    connect(pViewArtistAct,SIGNAL(triggered()),SLOT(OnViewArtistAction()));
     m_pTrackInfoView->addAction(pViewArtistAct);
 }
 
@@ -210,7 +211,13 @@ void QueueView::OnPlaybackStateChangeRequest(enum pb_state_e requestedState)
    //request received via libplayback asking for a playback state change
     if(requestedState==PB_STATE_STOP)
     { //request to move to stopped state, pause current playback
-        pausePlayback(); //pause playback
+        pauseResumePlayback(); //pause playback
+        m_playbackStateChangeReq = true; //indicate that playback state has been changed
+    }
+    else if(requestedState==PB_STATE_PLAY)
+    {
+        //request to resume playback
+        pauseResumePlayback(); //resume playback
         m_playbackStateChangeReq = true; //indicate that playback state has been changed
     }
 }
@@ -228,6 +235,7 @@ void QueueView::setQueueInitialState(QList<Spotify::Track*> queue)
     m_currQueuePos = 0; //initial playback pos
     m_currQueueBuffPos = 0; //initial buffering track pos
     m_shuffleStateChanged = false;
+    m_currTrackScrobbled = false;
 
     if(m_shuffle)
     { //shuffle mode active; shuffle the current play queue
@@ -262,7 +270,7 @@ void QueueView::clearViewSelection()
 
     QStandardItem* currTrackItem;
 
-    m_AlbumCoverContainer.clear();
+    m_AlbumCover.clear();
 
     m_pTrackInfoView->clearSelection();
 
@@ -359,9 +367,8 @@ void QueueView::loadTrackData(Spotify::Track* track)
     {
         if(track->getAlbum())
         {
-            m_pSession->getAlbumCover(track->getAlbum(),&m_CallerID); //request to load album coverart
+            m_pSession->loadImage(track->getAlbum()->getCover(),m_pAlbumCoverContainer); //request to load album coverart
         }
-
         m_currTrackDuration = track->getDuration().hour()*3600+track->getDuration().minute()*60+track->getDuration().second(); //current track duration (in secs)
         strCurrTrackDuration = track->getDurationAsString();
         outStream << "0:00" << " / " << strCurrTrackDuration;
@@ -414,11 +421,14 @@ void QueueView::OnTrackActivated(QModelIndex index)
     //A track has been activated (e.g., selected) in the track list
     //Display artist and album cover pic (if available)
 
-        Spotify::Error error;
-
         if(m_shuffleStateChanged)
             m_shuffleStateChanged=false;
 
+        if(m_shuffle)
+        { //re-shuffle the playqueue
+            shuffleQueue();
+        }
+
         QStandardItem* prevTrackItem = m_pTrackInfoModel->item(m_currQueuePos); //get previously selected item
         if(prevTrackItem)
             prevTrackItem->setData(false,ListViewDelegate::highlightItemTextRole); //remove highlight from prev. selected item
@@ -443,18 +453,30 @@ void QueueView::playTrack(Spotify::Track* track)
     //request to play (stream) specified track (and load associated track data)
 
     Spotify::Error error;
+    QString strArtist, strAlbum, strTrack, strAlbumArtist;
 
     if(m_playbackStateChangeReq)
     { //ask for state change
         m_playbackStateChangeReq = false;
         m_pOssoWrapper->requestPlaybackStateChange(PB_STATE_PLAY);
     }
+    m_currTrackScrobbled = false;
     m_pSession->resetPlaybackMembers();
     loadTrackData(track);
     m_PauseResumeBtn.setChecked(true); //toggle playing state
     m_pProgressSlider->setPos1(0);
     m_pProgressSlider->setPos2(0);
     setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true); //display progress indicator
+    strTrack = track->getName();
+    if(track->getArtist())
+        strArtist = track->getArtist()->getName();
+    if(track->getAlbum())
+    {
+        strAlbum = track->getAlbum()->getName();
+        if(track->getAlbum()->getArtist())
+            strAlbumArtist = track->getAlbum()->getArtist()->getName();
+    }
+    emit updateNowPlaying(strTrack,strArtist,strAlbum,strAlbumArtist); //send update now playing request to Last.fm
     QMaemo5InformationBox::information(this,tr("Buffering"));
     error = m_pSession->play(track); //request to start playback of track
     if(error.isError())
@@ -494,7 +516,15 @@ void QueueView::OnPlaybackPositionUpdated(qint64 playbackPos)
 
     QTextStream outStream(&strCurrTrackProgressInfo);
 
-    QTime currPlaybackPos(playbackPos/3600000,playbackPos/60000,(playbackPos/1000)%60,0);
+    int seconds = ( playbackPos / 1000);
+    QTime currPlaybackPos = QTime( 0, 0, 0, 0 );
+    currPlaybackPos = currPlaybackPos.addSecs( seconds );
+
+    if(!m_currTrackScrobbled && playbackPos>=240000)
+    {//scrobble track in case not already scrobbled (and track has played for at least 240 secs)
+       emit scrobbleTrack();
+       m_currTrackScrobbled = true;
+    }
 
     if(m_pCurrTrack)
     { //set track progress info in format current playback position / total track duration
@@ -526,18 +556,24 @@ void QueueView::OnPauseResumeToggle()
             m_playbackStateChangeReq = false;
             m_pOssoWrapper->requestPlaybackStateChange(PB_STATE_PLAY);
         }
-        m_pSession->pausePlayback(); //pause / resume playback of current track
+        m_pSession->pauseResumePlayback(); //pause / resume playback of current track
     }
 }
 
-void QueueView::pausePlayback()
+void QueueView::pauseResumePlayback()
 {
-    //request to pause music playback (and set the UI to corresponding state)
+    //request to pause / resume music playback (and set the UI to corresponding state)
+
+    m_pSession->pauseResumePlayback(); //pause playback
+
     if(m_playing)
     {
-        m_pSession->pausePlayback(); //pause playback
         m_PauseResumeBtn.setChecked(false); //set to paused state
     }
+    else
+    {
+        m_PauseResumeBtn.setChecked(true); //set to playing state
+    }
 }
 
 
@@ -562,6 +598,7 @@ void QueueView::OnNextTrackBtn()
         else
         { //no tracks left in shuffle queue
             m_currQueuePos = m_PlayQueue.size()-1;
+            m_currQueueBuffPos = m_PlayQueue.size()-1;
         }
     }
 
@@ -607,13 +644,9 @@ void QueueView::OnPrevTrackBtn()
             m_currShufflePos--;
             m_currQueuePos = m_ShuffleQueue.at(m_currShufflePos);
         }
-        else
-        { //no tracks left in shuffle queue
-            m_currQueuePos = -1;
-        }
     }
 
-    if((!m_shuffle && (m_currQueuePos > 0))||(m_shuffle && (m_currQueuePos>=0)))
+    if((!m_shuffle && (m_currQueuePos > 0))||(m_shuffle && (m_currQueuePos>0)))
     {
         if(!m_shuffle)
         {
@@ -644,10 +677,20 @@ void QueueView::OnTrackPlaybackFinished()
 
     qDebug() << "Playback finished..";
 
+    QString strTrack, strArtist, strAlbum, strAlbumArtist;
+
     Spotify::Error error;
 
     qint32 oldQueuePos = m_currQueuePos; //save current (old) queue position
-    qint32 oldShufflePos = m_currShufflePos;
+
+    if(!m_currTrackScrobbled && m_currTrackDuration>=30)
+    {//scrobble track in case not already scrobbled (and track duration is at least 30secs)
+       emit scrobbleTrack();
+    }
+    else
+    {
+        m_currTrackScrobbled = false; //reset current track scrobbled status
+    }
 
     if(m_shuffle)
     { //shuffle mode active, get position of next track to play from shuffle queue
@@ -679,7 +722,17 @@ void QueueView::OnTrackPlaybackFinished()
             QModelIndex index = m_pTrackInfoModel->index(m_currQueuePos,0);
             if(index.isValid())
                 m_pTrackInfoView->setCurrentIndex(index);
-            loadTrackData(m_pCurrTrack);
+            loadTrackData(m_pCurrTrack); //load track info
+            strTrack = m_pCurrTrack->getName();
+            if(m_pCurrTrack->getArtist())
+                strArtist = m_pCurrTrack->getArtist()->getName();
+            if(m_pCurrTrack->getAlbum())
+            {
+                strAlbum = m_pCurrTrack->getAlbum()->getName();
+                if(m_pCurrTrack->getAlbum()->getArtist())
+                    strAlbumArtist = m_pCurrTrack->getAlbum()->getArtist()->getName();
+            }
+            emit updateNowPlaying(strTrack,strArtist,strAlbum,strAlbumArtist); //send update now playing request to Last.fm
             if(!m_pSession->getPreBufferingActive()) //pre-buffering is not active, buffering / playback should be initiated
             {
                 m_pSession->resetPlaybackMembers();
@@ -694,7 +747,6 @@ void QueueView::OnTrackPlaybackFinished()
                 m_pSession->setPlaybackPos(0); //reset current playback position
                 if(m_currQueueBuffPos==-1)
                     m_pProgressSlider->setPos1(SLIDER_WIDTH); //set buffering progress to 100% (final track buffered completely)
-                qDebug() << m_currQueueBuffPos << "," << oldQueuePos;
                 if(m_currQueueBuffPos != oldQueuePos)
                 {
                     if(m_shuffleStateChanged)
@@ -722,15 +774,13 @@ void QueueView::OnTrackPlaybackFinished()
     }
 }
 
-void QueueView::OnAlbumCoverReady(QImage image, qint32 callerID)
+void QueueView::OnAlbumCoverLoaded()
 {
     //Album coverart has been loaded (for currently selected track),
     //assign it to the album coverart container.
 
-    if(callerID==m_CallerID)
-    {
-        m_AlbumCoverContainer.setPixmap(QPixmap::fromImage(image).scaled(355,355));
-    }
+    if(m_pAlbumCoverContainer)
+        m_AlbumCover.setPixmap(QPixmap::fromImage(m_pAlbumCoverContainer->getImage()).scaled(QUEUEVIEW_COVERART_WIDTH,QUEUEVIEW_COVERART_HEIGHT));
 }
 
 void QueueView::OnShuffleToggle()
@@ -839,7 +889,7 @@ void QueueView::OnAlbumBrowseReady(Spotify::AlbumBrowser* browser, qint32 caller
                     QMaemo5InformationBox::information(this,tr("Failed to load tracks!"));
                 }
                 browser->sortByDiscIndex(); //sort tracks in album order
-                setQueueInitialState(browser->getTracks(false)); //only add available (playable) album tracks to the playqueue
+                emit playQueueSet(browser->getTracks(false)); //only add available (playable) album tracks to the playqueue
                 setListViewState();
             }
         }
@@ -862,3 +912,18 @@ void QueueView::OnViewAlbumAction()
         }
     }
 }
+
+void QueueView::OnViewArtistAction()
+{
+    //Request to show Artist View for currently selected artist.
+
+    Spotify::Track* track = m_PlayQueue.at(m_pTrackInfoView->currentIndex().row());
+
+    if(track)
+    {
+        if(track->getArtist())
+        {
+            emit showArtistView(track->getArtist());
+        }
+    }
+}
