@@ -102,6 +102,7 @@ private slots:
     void OnMenubarAction(QAction* action);
     void OnViewAlbumAction();
     void OnPlayAlbumAction();
+    void OnViewArtistAction();
     void OnAlbumBrowseReady(Spotify::AlbumBrowser* browser, qint32 callerID);
 
 signals:
@@ -109,6 +110,7 @@ signals:
     void playQueueSet(const QList<Spotify::Track*>&); //signal emitted upon new playqueue set
     void showFMTransmitterDialog(); //signal emitted upon request to show the FM Transmitter control panel dialog (plugin)
     void showAlbumView(Spotify::Album*); //request to show the album view for the specified album
+    void showArtistView(Spotify::Artist*); //request to show the artist view for the specified artist
 };
 
 #endif // TRACKVIEW_H
