@@ -30,14 +30,16 @@
 #include "TrackView.h"
 #include "AlbumView.h"
 #include "QueueView.h"
+#include "artistview.h"
 #include "SpectrumAnalyzer.h"
 #include "customlabel.h"
 #include "OssoWrapper.h"
+#include "lastfmscrobbler.h"
 
 
 //defines
 
-#define APP_VERSION "0.1.0-7" //application version string
+#define APP_VERSION "0.2.0-1" //application version string
 
 #define PLAYLIST_LOADING_TIMEOUT  30 //timeout before giving up on waiting for playlists to load (in seconds)
 
@@ -66,16 +68,20 @@ private:
 
     SettingsDialog* m_pSettingsDlg;
 
-    TrackView* m_pTrackView;
+    TrackView* m_pTrackView; //artist view instance
 
-    AlbumView* m_pAlbumView;
+    AlbumView* m_pAlbumView; //album view instance
 
     QueueView* m_pQueueView; //playqueue (Now Playing) view instance
 
+    ArtistView* m_pArtistView; //artist view instance
+
     Spotify::Session* m_pSession; //main Spotify session instance (handles communication with Spotify backend)
 
     OssoWrapper* m_pOssoWrapper; //wrapper instance for libosso
 
+    LastFMScrobbler* m_pLastFMScrobbler; //Last.fm scrobbling instance
+
     QPushButton m_SearchBtn;
 
     QLineEdit m_SearchInput;
@@ -120,6 +126,8 @@ private:
 
     void initAboutDialog(); //setup the application about dialog
 
+    void setupInitialState(); //set up the app initial state (audio etc.)
+
 
 
 private slots:
@@ -141,7 +149,11 @@ private slots:
      void OnSettingsSaved();
      void OnPlaylistLoadingTimeout();
      void OnDisplayFMTransmitterDialog();
-     void OnShowAlbumView(Spotify::Album* album);
+     void OnShowAlbumView(Spotify::Album* pAlbum);
+     void OnShowArtistView(Spotify::Artist* pArtist);
+     void OnScrobblingStatus(bool status, const QString& error);
+     void OnScrobbleUpdateNowPlaying(const QString& strTrackName, const QString& strArtistName, const QString& strAlbumName, const QString& strAlbumArtist); //update Last.fm now playing info
+     void OnScrobbleTrack(); //scrobble current track to Last.fm (and send scrobbles in queue)
 
 signals:
 
